==> netrw-0.5.tar.bz2: netrw-0.5/netread.c <==
/* $Id: netread.c,v 1.4 2002/07/08 14:17:29 xdenemar Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define BUFFSIZE 1<<14

void usage(void)
{
    fprintf(stderr,
        "Usage: netread [udp] <options> <port>\n"
        "\n"
        "Default TCP protocol can be changed to UDP by ``udp'' argument.\n"
        "UDP options\n"
        "  currently none\n"
        "TCP options\n"
        "  -f <host>   firewall mode, connection is initiated by netread\n"
        "general options\n"
        "  -v          verbose\n"
        "  -V          version\n");
    exit(1);
}

int main(int argc, char **argv)
{
    int udp = 0;
    int opt;
    int verbose = 0;
    int firewall = 0;
    char *host_str = NULL;
    struct hostent *host;
    char *service;
    struct servent *se;
    unsigned short port;
    struct sockaddr_in sin;
    struct sockaddr_in r_addr;
    int sock;
    unsigned int addrsize = sizeof(r_addr);
    char buf[BUFFSIZE];
    ssize_t count;
  
    if (argc < 2)
        usage();

    if (strcmp(argv[1], "udp") == 0) {
        udp = 1;
        argv[1] = argv[0];
    }

    while ((opt = getopt(argc - udp, argv + udp, "f:vV")) != -1) {
        switch (opt) {
        case 'f':           /* firewall mode */
            if (udp)
                usage();
            firewall = 1;
            host_str = optarg;
            break;
        case 'v':           /* verbose */
            verbose = 1;
            break;
        case 'V':           /* version */
            printf("netrw tools version %s\n%s\n", VERSION,
                   "$Id: netread.c,v 1.4 2002/07/08 14:17:29 xdenemar Exp $");
            return 0;
            break;
        case '?':           /* bad option */
            usage();
        }
    }

    if ((optind + udp) >= argc
        || (service = argv[optind + udp]) == NULL)
        usage();

    if ((se = getservbyname(service, (udp ? "udp" : "tcp")))) {
        port = se->s_port;
    }
    else {
        /* service may also be a port number */
        char *endptr = NULL;
        port = htons(strtol(service, &endptr, 0));
        if (*service == '\0' || *endptr != '\0') {
            fprintf(stderr, "%s/%s: no such service\n",
                    service, (udp ? "udp" : "tcp"));
            return 1;
        }
    }

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = port;

    /* create socket */
    if (udp) {
        if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (bind(sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("bind()");
            return 1;
        }
    }
    else if (firewall) {
        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (!(host = gethostbyname(host_str))) {
            perror("gethostbyname()");
            return 1;
        }
  
        sin.sin_addr = **(struct in_addr **)host->h_addr_list;
        if (connect(sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("connect()");
            return 1;
        }

        if (verbose) {
          fprintf(stderr, "Connected to %s:%d\n",
                  inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
        }
    }
    else {
        int listen_sock;

        if ((listen_sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (bind(listen_sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("bind()");
            return 1;
        }
        if (listen(listen_sock, 0)) {
            perror("listen()");
            return 1;
        }
        if (verbose)
            fprintf(stderr, "Listening on %d\n", ntohs(port));

        if ((sock = accept(listen_sock, (struct sockaddr *) &r_addr,
                           &addrsize)) == -1) {
            perror("accept()");
            return 1;
        }
        if (verbose) {
            fprintf(stderr, "Accepted connection from %s:%d\n",
                    inet_ntoa(r_addr.sin_addr), ntohs(r_addr.sin_port));
        }
        close(listen_sock);
    }

#define read_sock(sock, buf, r_addr, size) \
    (udp ? recvfrom(sock, buf, BUFFSIZE, 0, r_addr, size) \
         : read(sock, buf, BUFFSIZE))

    while ((count = read_sock(sock, buf, (struct sockaddr *) &r_addr,
                              &addrsize)) > 0) {
        int wr;
        char *p;

        if (udp && verbose) {
            fprintf(stderr, "Message from %s:%d\n",
                    inet_ntoa(r_addr.sin_addr), ntohs(r_addr.sin_port));
        }

        for (p = buf; (wr = write(STDOUT_FILENO, p, count)) > 0; p += wr) {
            if (!(count -= wr))
                break;
        }
        if (wr < 0) {
            perror("write()");
            return 1;
        }
    }

    if (count < 0) {
        perror("read_sock()");
        return 1;
    }

    close(sock);
    return 0;
}

==> netrw-0.7.tar.bz2: netrw-0.7/netread.c <==
/* $Id: netread.c,v 1.6 2002/07/22 08:23:09 jirka Exp $ */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "common.h"

void usage(void)
{
    fprintf(stderr,
        "Usage: netread [udp] <options> <port>\n"
        "\n"
        "Default TCP protocol can be changed to UDP by ``udp'' argument.\n"
        "UDP options\n"
        "  currently none\n"
        "TCP options\n"
        "  -f <host>        firewall mode, connection is initiated by netread\n"
        "general options\n"
        HASH_USAGE
        VERBOSITY_USAGE
        "  -V               version\n");
    exit(1);
}

int main(int argc, char **argv)
{
    int udp = 0;
    int opt;
    int verbose = 0;
    int quiet = 0;
    int firewall = 0;
    char *host_str = NULL;
    struct hostent *host;
    char *service;
    struct servent *se;
    unsigned short port;
    struct sockaddr_in sin;
    struct sockaddr_in r_addr;
    int sock;
    unsigned int addrsize = sizeof(r_addr);
    char buf[BUFFSIZE];
    ssize_t count;
  
    if (argc < 2)
        usage();

    if (strcmp(argv[1], "udp") == 0) {
        udp = 1;
        argv[1] = argv[0];
    }

    while ((opt = getopt(argc - udp, argv + udp,
                         "f:V" HASH_OPT VERBOSITY_OPT)) != -1) {
        switch (opt) {
        case 'f':           /* firewall mode */
            if (udp)
                usage();
            firewall = 1;
            host_str = optarg;
            break;

        case 'V':           /* version */
            printf("netrw tools version %s\n%s\n", VERSION,
                   "$Id: netread.c,v 1.6 2002/07/22 08:23:09 jirka Exp $");
            return 0;
            break;

        HASH_CASE           /* hash printing options */
        VERBOSITY_CASE      /* verbosity options */

        case '?':           /* bad option */
            usage();
        }
    }

    if ((optind + udp) >= argc
        || (service = argv[optind + udp]) == NULL)
        usage();

    if ((se = getservbyname(service, (udp ? "udp" : "tcp")))) {
        port = se->s_port;
    }
    else {
        /* service may also be a port number */
        char *endptr = NULL;
        port = htons(strtol(service, &endptr, 0));
        if (*service == '\0' || *endptr != '\0') {
            fprintf(stderr, "%s/%s: no such service\n",
                    service, (udp ? "udp" : "tcp"));
            return 1;
        }
    }

    sin.sin_family = AF_INET;
    sin.sin_addr.s_addr = INADDR_ANY;
    sin.sin_port = port;

    /* create socket */
    if (udp) {
        if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (bind(sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("bind()");
            return 1;
        }
    }
    else if (firewall) {
        if ((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (!(host = gethostbyname(host_str))) {
            perror("gethostbyname()");
            return 1;
        }
  
        sin.sin_addr = **(struct in_addr **)host->h_addr_list;
        if (connect(sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("connect()");
            return 1;
        }

        if (verbose) {
          fprintf(stderr, "Connected to %s:%d\n",
                  inet_ntoa(sin.sin_addr), ntohs(sin.sin_port));
        }
    }
    else {
        int listen_sock;

        if ((listen_sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
            perror("sokcet()");
            return 1;
        }

        if (bind(listen_sock, (struct sockaddr *) &sin, sizeof(sin))) {
            perror("bind()");
            return 1;
        }
        if (listen(listen_sock, 0)) {
            perror("listen()");
            return 1;
        }
        if (verbose)
            fprintf(stderr, "Listening on %d\n", ntohs(port));

        if ((sock = accept(listen_sock, (struct sockaddr *) &r_addr,
                           &addrsize)) == -1) {
            perror("accept()");
            return 1;
        }
        if (verbose) {
            fprintf(stderr, "Accepted connection from %s:%d\n",
                    inet_ntoa(r_addr.sin_addr), ntohs(r_addr.sin_port));
        }
        close(listen_sock);
    }

    while ((count = read_sock(sock, buf, (struct sockaddr *) &r_addr,
                              &addrsize)) > 0) {
        int wr;
        char *p;

        if (udp && verbose) {
            fprintf(stderr, "Message from %s:%d\n",
                    inet_ntoa(r_addr.sin_addr), ntohs(r_addr.sin_port));
        }

        if (!quiet)
            print_hashes(count);

        for (p = buf; (wr = write(STDOUT_FILENO, p, count)) > 0; p += wr) {
            if (!(count -= wr))
                break;
        }
        if (wr < 0) {
            perror("write()");
            return 1;
        }
    }

    if (!quiet)
        print_summary();

    if (count < 0) {
        perror("read_sock()");
        return 1;
    }

    close(sock);
    return 0;
}
